# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2014-2015 E-MIPS (http://www.e-mips.com.ar).
#    All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General
#    Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
import netsvc

from openerp.osv import fields, orm, osv
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT
from openerp.tools.translate import _


class stock_location(osv.osv):
    _name = 'stock.location'
    _inherit = 'stock.location'

    _columns = {
        'id_ubicacion': fields.integer("Id CM")
    }


class split_in_production_lot(osv.osv_memory):
    _name = "stock.move.split"
    _inherit = "stock.move.split"

    def split(self, cr, uid, ids, move_ids, context=None):
        res = super(split_in_production_lot, self).split(
            cr, uid, ids, move_ids, context=context)
        component_obj = self.pool['service.component']
        move_obj = self.pool['stock.move']
        moves = move_obj.browse(cr, uid, list(set(move_ids + res)), context)
        for move in moves:
            move.refresh()
            if move.picking_id.type != 'out':
                continue
            comp_id = component_obj.search(
                cr, uid, [('sale_order_line', '=', move.sale_line_id.id)])
            if not comp_id or len(comp_id) != 1:
                continue
            write_val = {
                'prodlot_id': move.prodlot_id.id,
            }
            component_obj.write(cr, uid, comp_id, write_val, context=context)
        return res


class stock_picking(osv.osv):
    _name = 'stock.picking'
    _inherit = 'stock.picking'

    def do_partial(self, cr, uid, ids, partial_datas, context=None):

        material_obj = self.pool.get('service.order.material.line')

        from_service = context.get('from_service', False)
        material_ids = material_obj.search(
            cr, uid, [('picking_id', 'in', ids)], context=context)
        if len(material_ids) and not from_service:
            raise osv.except_osv(
                _("Stock Error!"),
                _("This picking is associated with " +
                  "a Service Order. Please, inform materials from there"))

        res = super(stock_picking, self).do_partial(
            cr, uid, ids, partial_datas, context=context)

        # BAD HACK to fix state 'assigned' in new stock.moves
        picking_obj = self.pool['stock.picking']
        move_obj = self.pool['stock.move']
        if type(res) != dict or not res.keys():
            return res

        original_picking_id = res.keys()[0]
        if 'delivered_picking' not in res[original_picking_id]:
            return res
        original_picking = picking_obj.browse(cr, uid, original_picking_id,
                                              context)
        new_picking = picking_obj.browse(
            cr, uid, res[original_picking_id]['delivered_picking'], context)
        if original_picking.type != 'out':
            return res
        if new_picking.state == 'assigned':
            wf_service = netsvc.LocalService("workflow")
            wf_service.trg_validate(
                uid, 'stock.picking', new_picking.id, 'picking_confirm', cr)
        return res

    # Antoher BAD HACK (Override method completely only to keep prodlots
    # on the stock.moves that remain in the original picking
    def do_partial_no_confirm(self, cr, uid, ids, partial_datas,
                              context=None):
        """ Makes partial picking and moves but does not put them in done state.
        Code is directly taken form OpenERP source code and not refactored.
        @param partial_datas : Dictionary containing details of partial picking
                          like partner_id, partner_id, delivery_date,
                          delivery moves with product_id, product_qty, uom
        @return: Dictionary of values
        """
        if context is None:
            context = {}
        else:
            context = dict(context)
        res = {}
        move_obj = self.pool.get('stock.move')
        uom_obj = self.pool.get('product.uom')
        sequence_obj = self.pool.get('ir.sequence')
        wf_service = netsvc.LocalService("workflow")
        for pick in self.browse(cr, uid, ids, context=context):
            new_picking = None
            complete, too_many, too_few = [], [], []

            move_product_qty, prodlot_ids = {}, {}
            product_uoms, partial_qty = {}, {}

            for move in pick.move_lines:
                # if move.state in ('done', 'cancel'):
                #     continue
                partial_data = partial_datas.get('move%s' % (move.id), {})
                product_qty = partial_data.get('product_qty', 0.0)
                move_product_qty[move.id] = product_qty
                product_uom = partial_data.get('product_uom', False)
                prodlot_id = partial_data.get('prodlot_id')
                prodlot_ids[move.id] = prodlot_id
                product_uoms[move.id] = product_uom
                partial_qty[move.id] = uom_obj._compute_qty(
                    cr, uid, product_uoms[move.id],
                    product_qty, move.product_uom.id)
                if move.product_qty == partial_qty[move.id]:
                    complete.append(move)
                elif move.product_qty > partial_qty[move.id]:
                    too_few.append(move)
                else:
                    too_many.append(move)
            # We remove section of code that manage incoming picking
            # please refere to original code if needed
            for move in too_few:
                product_qty = move_product_qty[move.id]
                if not new_picking:
                    new_picking_name = pick.name
                    self.write(cr, uid, [pick.id], {
                        'name': sequence_obj.get(
                            cr, uid, 'stock.picking.%s' % pick.type),
                    })
                    new_picking = self.copy(cr, uid, pick.id,
                                            {
                                                'name': new_picking_name,
                                                'move_lines': [],
                                                'state': 'draft',
                                            })
                if product_qty != 0:
                    defaults = {
                        'product_qty': product_qty,
                        # TODO: put correct uos_qty
                        'product_uos_qty': product_qty,
                        'picking_id': new_picking,
                        'state': move.state,
                        'move_dest_id': False,
                        'price_unit': move.price_unit,
                        'product_uom': product_uoms[move.id]
                    }
                    prodlot_id = prodlot_ids[move.id]
                    if prodlot_id:
                        defaults.update(prodlot_id=prodlot_id)
                    move_obj.copy(cr, uid, move.id, defaults)
                move_obj.write(cr, uid, [move.id],
                               {
                    'product_qty': move.product_qty - partial_qty[move.id],
                    # TODO: put correct uos_qty
                    'product_uos_qty': move.product_qty - partial_qty[move.id],
                    'prodlot_id': move.prodlot_id.id,
                    'tracking_id': False,
                })

            if new_picking:
                move_obj.write(
                    cr, uid, [c.id for c in complete], {
                        'picking_id': new_picking
                    })
            for move in complete:
                defaults = {'product_uom': product_uoms[move.id],
                            'product_qty': move_product_qty[move.id]}
                if prodlot_ids.get(move.id):
                    defaults.update({'prodlot_id': prodlot_ids[move.id]})
                move_obj.write(cr, uid, [move.id], defaults)
            for move in too_many:
                product_qty = move_product_qty[move.id]
                defaults = {
                    'product_qty': product_qty,
                    # TODO: put correct uos_qty
                    'product_uos_qty': product_qty,
                    'product_uom': product_uoms[move.id]
                }
                prodlot_id = prodlot_ids.get(move.id)
                if prodlot_ids.get(move.id):
                    defaults.update(prodlot_id=prodlot_id)
                if new_picking:
                    defaults.update(picking_id=new_picking)
                move_obj.write(cr, uid, [move.id], defaults)

            # At first we confirm the new picking (if necessary)
            # We have removed here confirmation workflow calls
            if new_picking:
                wf_service.trg_validate(
                    uid, 'stock.picking', new_picking, 'button_confirm', cr)
                self.write(cr, uid, [pick.id], {'backorder_id': new_picking})
                delivered_pack_id = new_picking
            else:
                delivered_pack_id = pick.id

            delivered_pack = self.browse(
                cr, uid, delivered_pack_id, context=context)
            res[pick.id] = {'delivered_picking': delivered_pack.id or False}

        return res

    def draft_validate(self, cr, uid, ids, context=None):
        # Chequeamos que si es a traves de una
        # service order, se debe hacer por informar materiales
        material_obj = self.pool.get('service.order.material.line')

        from_service = context.get('from_service', False)
        material_ids = material_obj.search(
            cr, uid, [('picking_id', 'in', ids)], context=context)
        if len(material_ids) and not from_service:
            raise osv.except_osv(
                _("Stock Error!"),
                _("This picking is associated with " +
                  "a Service Order. Please, inform materials from there"))

        return super(stock_picking, self).draft_validate(cr, uid, ids, context)


class stock_move(osv.osv):
    _name = 'stock.move'
    _inherit = 'stock.move'

    def write(self, cr, uid, ids, vals, context=None):
        for move in self.browse(cr, uid, ids, context):
            if 'prodlot_id' in vals:
                component_obj = self.pool['service.component']
                comp_ids = component_obj.search(cr, uid, [
                    ('move_id', '=', move.id)],
                    context=context)
                if not comp_ids:
                    continue
                vals = {
                    'prodlot_id': vals['prodlot_id'],
                }
                component_obj.write(cr, uid, comp_ids, vals, context=context)
        return super(stock_move, self).write(cr, uid, ids, vals,
                                             context=context)

    def delete_prodlot(self, cr, uid, ids, context=None):
        move_obj = self.pool['stock.move']
        for move in self.browse(cr, uid, ids):
            if not move.prodlot_id.id:
                continue
            move_vals = {
                'prodlot_id': False,
            }
            move_obj.write(cr, uid, [move.id], move_vals,
                           context=context)
        return True

    def action_cancel(self, cr, uid, ids, context=None):
        material_obj = self.pool.get('service.order.material.line')

        material_obj = self.pool.get('service.order.material.line')
        material_ids = material_obj.search(
            cr, uid, [('move_id', 'in', ids)], context=context)

        # Cancelamos las lineas materiales
        # correspondientes a partes de servicio
        material_obj.write(cr, uid, material_ids,
                           {'state': 'cancel'}, context=context)
        return super(stock_move, self).action_cancel(cr, uid, ids, context)

    def action_done(self, cr, uid, ids, context=None):

        if not context:
            context = {}

        from_service = context.get('from_service', False)
        material_obj = self.pool.get('service.order.material.line')

        material_obj = self.pool.get('service.order.material.line')
        material_ids = material_obj.search(cr, uid, [('move_id', 'in', ids)],
                                           context=context)
        if len(material_ids) and not from_service:
            raise osv.except_osv(
                _("Stock Error!"),
                _("This move is associated with a Service Order. " +
                  "Please, inform materials from there"))

        return super(stock_move, self).action_done(cr, uid, ids,
                                                   context=context)


class suitcase_procure_materials(orm.Model):
    _name = "suitcase.procure.materials"
    _description = "Requested Materials for Suitcase"

    _columns = {
        'suitcase_id': fields.many2one('stock.suitcase', "Suitcase"),
        'product_id': fields.many2one('product.product', 'Product',
                                      required=True),
        'product_qty': fields.float("Product Qty", required=True),
        'state': fields.selection([('draft', 'Requested'),
                                   ('done', 'Done')], 'State', readonly=True),
    }

    _defaults = {
        'state': 'draft',
    }


class stock_suitcase(orm.Model):
    _name = 'stock.suitcase'
    _description = 'Stock Suitcase'

    _columns = {
        'name': fields.char('Suitcase Name', size=64, required=True),
        'employee_id': fields.many2one('hr.employee', 'Employee'),
        'location_id': fields.many2one('stock.location', 'Location'),
        'state': fields.selection([('draft', 'Draft'),
                                   ('available', 'Available'),
                                   ('assigned', 'Assigned')], 'State',
                                  readonly=True),
        'active': fields.boolean('Active'),
        'personal': fields.boolean('Personal'),
        'current_materials_ids': fields.one2many(
            'suitcase.current.materials', 'suitcase_id', 'Current materials'),
        'waiting_materials_ids': fields.one2many(
            'suitcase.waiting.materials', 'suitcase_id', 'Waiting materials'),
        'requested_materials_ids': fields.one2many(
            'suitcase.procure.materials', 'suitcase_id', 'Requested Materials',
            domain=[('state', '=', 'draft')])
    }

    _defaults = {
        'active': True,
        'state': 'draft',
    }

    def create_picking(self, cr, uid, suitcase_id, material_lines,
                       sense='in', src_doc_tmp='', context=None):
        """
        Creates picking and moves.
        @param suitcase_id: integer or long. Just *ONE* id.
        @param sense: sense of picking, 'in' means dest_loc_id is the suitcase,
                      'out' means src_loc_id is the suitcase.
        @param src_doc_tmp: template of the string to use as moves' name.
                            Mapping attrs allowed are user
                            (resolves to user.name),
                            suitcase (resolves to suitcase.name) and
                            date (current date).
        @param materials_lines: expect list of suitcase.add.materials.line
                                records when sense is.
        @return: id of the created picking.
        @return type: integer (or long).
        """
        if isinstance(suitcase_id, list):
            suitcase_id = suitcase_id[0]

        suitcase = self.browse(cr, uid, suitcase_id, context=context)
        user = self.pool.get("res.users").browse(cr, uid, uid, context=context)
        picking_obj = self.pool.get("stock.picking")
        date = time.strftime(DEFAULT_SERVER_DATETIME_FORMAT)
        source_doc_data = {
            "user": user.name,
            "suitcase": suitcase.name,
            "date": date
        }

        if not src_doc_tmp:
            src_doc_tmp = _("Materials added to %(suitcase)s on " +
                            "%(date)s by %(user)s")
        source_doc = src_doc_tmp % source_doc_data
        suitcase_loc_id = suitcase.location_id.id
        type = "internal"

        new_picking_data = {"origin": source_doc}
        if sense == 'in':
            new_picking_data["location_dest_id"] = suitcase_loc_id
            for material in material_lines:
                new_move_data = {
                    "type": type,
                    "location_dest_id": suitcase_loc_id,
                    "name": source_doc,
                    "product_id": material.product_id.id,
                    "product_uom": material.product_uom_id.id,
                    "product_qty": material.product_uom_qty,
                    "location_id": material.location_id.id,
                }
                new_move = (0, False, new_move_data)
                new_picking_data.setdefault("move_lines", []).append(new_move)
        # when removing materials from a suitcase.
        elif sense == 'out' and material_lines:
            new_picking_data["location_id"] = suitcase_loc_id
            for material in material_lines:
                new_move_data = {
                    "type": type,
                    "location_dest_id": material.location_id.id,
                    "name": source_doc,
                    "product_id": material.product_id.id,
                    "product_uom": material.product_uom_id.id,
                    "product_qty": material.product_uom_qty,
                    "location_id": suitcase_loc_id,
                }
                new_move = (0, False, new_move_data)
                new_picking_data.setdefault("move_lines", []).append(new_move)

        new_picking_id = picking_obj.create(cr, uid, new_picking_data)
        picking_obj.draft_force_assign(cr, uid, [new_picking_id])
        return new_picking_id

    def unlink(self, cr, uid, ids, context=None):
        for s in self.browse(cr, uid, ids, context=context):
            if s.state != 'draft':
                raise osv.except_osv(
                    _("Cannot delete!"),
                    _("You cannot delete a Suitcase that is not in draft. " +
                      "If you want, deactivate it."))

        return super(stock_suitcase, self).unlink(cr, uid, ids, context)

    def confirm(self, cr, uid, ids, context=None):
        # Creamos la location asociada a la valija
        for suitcase in self.browse(cr, uid, ids, context=context):
            location_vals = {
                'name': suitcase.name,
                'usage': 'internal',
            }

            location_id = self.pool.get('stock.location').create(
                cr, uid, location_vals, context=context)
            self.write(cr, uid, suitcase.id,
                       {
                           'location_id': location_id,
                           'state': 'available'
                       }, context=context)

        return True
