from osv import osv, fields
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT as DSDTF
from datetime import datetime


# Service Wait
class service_wait(osv.osv):
    _name = "service.wait"
    _description = "Wait"
    _order = "date_start desc"

    def _get_actions_changed_type(self, cr, uid, ids, context=None):
        result = {}
        for order in self.pool.get('service.order').browse(
                cr, uid, ids, context=context):
            for action in order.action_ids:
                result[action.id] = True
        return result.keys()

    _columns = {
        'name': fields.char('Description', size=256),
        'reason': fields.many2one('service.wait.reason', 'Reason'),
        'user_id': fields.many2one('res.users', 'User'),
        'date_start': fields.datetime('Date start'),
        'date_end': fields.datetime('Date end'),
        'customer_wait': fields.related(
            'reason', 'customer_wait', type='boolean',
            string="Customer Wait", store=True, readonly=True),
        'state': fields.selection(
            [('waiting', 'Waiting'),
             ('finished', 'Finished')],
            'State', readonly=True, select=True),
        'order_id': fields.many2one('service.order', 'Service Order'),
        'partner_id': fields.related('order_id', 'partner_id',
                                     string="Partner", type="many2one",
                                     relation="res.partner", store=True),
        'order_type': fields.related(
            'order_id', 'type', string="Order type",
            type="many2one", relation="service.order.type",
            store={
                'service.order': (_get_actions_changed_type,
                                  ['type', 'action_ids'], 10),
            }),
    }
    _defaults = {
        'state': 'waiting',
        'date_start': fields.datetime.now,
        'user_id': lambda self, cr, uid, context: uid,
    }

    def mark_finished(self, cr, uid, ids, context=None):
        write_vals = {
            'state': 'finished',
            'date_end': datetime.strftime(datetime.today(), DSDTF)
        }
        self.write(cr, uid, ids, write_vals, context)


class service_wait_reason(osv.osv):
    _name = "service.wait.reason"
    _description = "Wait Reason"

    _columns = {
        'name': fields.char('Reason', size=64),
        'customer_wait': fields.boolean('Customer Wait')
    }

    _defaults = {
        'customer_wait': lambda *x: False,
    }
