# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2014 E-MIPS/Eynes
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.addons import decimal_precision as dp
from openerp.osv import fields, orm, osv
from openerp.tools.translate import _


class suitcase_add_materials(orm.TransientModel):
    """
    Wizard to add materials to the suitcase
    """
    _name = "suitcase.add.materials.wizard"
    _description = "Suitcase Add Materials"

    _columns = {
        'material_line_ids': fields.one2many('suitcase.add.materials.line', 'wiz_id', 'Lines'),
        'picking_id': fields.many2one('stock.picking', 'Picking'),
        'state': fields.selection([("draft", "Draft"), ("done", "Done")], 'State'),
    }

    _defaults = {
        'state': 'draft',
    }

    def fields_view_get(self, cr, uid, view_id, view_type='form', context=None, toolbar=False):
        if context is None:
            context = {}

        res = super(suitcase_add_materials, self).fields_view_get(cr, uid, view_id, view_type,
                                                                  context=context, toolbar=toolbar)
        active_id = context.get("active_id", False)
        if not active_id:
            return res

        if context.get("from_kanban_view", False):
            # if method is called from kanban view active_id is the id of
            # suitcase.current.materials object, so let's look for the suitcase_id
            cm = self.pool.get("suitcase.current.materials").browse(cr, uid, active_id)
            active_id = cm.suitcase_id.id

        suitcase = self.pool.get('stock.suitcase').browse(cr, uid, active_id, context=context)
        location_id = suitcase.location_id.id
        tree_lines = res['fields']['material_line_ids']['views']['tree']

        dom_loc = ('id', '!=', location_id)
        if not dom_loc in tree_lines['fields']['location_id']['domain']:
            tree_lines['fields']['location_id']['domain'].append(dom_loc)

        if context.get("sense", "in") == "out":
            dom = tuple([line.product_id.id for line in suitcase.current_materials_ids])
            if dom:
                tree_lines['fields']['product_id']['domain'] = [('id', 'in', dom)]
        else:
            tree_lines['fields']['product_id']['domain'] = []

        return res

    def default_get(self, cr, uid, fields, context=None):
        suitcase_obj = self.pool.get('stock.suitcase')
        material_line_obj = self.pool.get('suitcase.add.materials.line')

        if context is None: context = {}
        res = super(suitcase_add_materials, self).default_get(cr, uid,
                fields, context=context)

        suitcase_ids = context.get('active_ids', [])
        if not suitcase_ids or not context.get('active_model') == 'stock.suitcase':
            return res

        # Buscamos los requested_materials de la valija
        suitcase = suitcase_obj.browse(cr, uid, suitcase_ids[0], context)
        materials = []
        for requested in suitcase.requested_materials_ids:

            change_vals = material_line_obj.onchange_product_id(cr, uid, [],
                    requested.product_id.id, context=context)

            def_vals = material_line_obj.default_get(cr, uid, ['location_id', 'warehouse_id'], context=context)
            vals = {
                'product_id': requested.product_id.id,
                'product_uom_id': change_vals['value']['product_uom_id'],
                'product_uom_qty': requested.product_qty,
                'request_id': requested.id,
            }
            vals.update(def_vals)
            materials.append((0, 0, vals))

        res['material_line_ids'] = materials
        return res


    def confirm_materials(self, cr, uid, ids, context=None):
        """
        Add materials to the suitcase and create the stock.picking associated
        """
        material_line_obj = self.pool.get('suitcase.procure.materials')
        if context is None:
            context = {}

        active_id = context.get("active_id", False)
        if not active_id:
            # No está guardada la suitcase ? lanzar un mensaje de error ?
            return

        # if adding materials from the kanban view
        if context.get("from_kanban_view", False):
            cm = self.pool.get("suitcase.current.materials").browse(cr, uid, active_id)
            active_id = cm.suitcase_id.id

        sense = "in"
        src_doc_tmp = ''
        if context.get("sense", "in") == "out":
            sense = "out"
            src_doc_tmp = "Materials removed from %(suitcase)s"

        suitcase = self.pool.get('stock.suitcase').browse(cr, uid, active_id)
        this = self.browse(cr, uid, ids[0], context=context)
        lines = this.material_line_ids
        new_picking_id = suitcase.create_picking(sense=sense, src_doc_tmp=src_doc_tmp,
                                                 material_lines=lines)
        if not new_picking_id:
            osv.except_osv("Error!", _("Couldn't create picking! Check the logs for more info"))
            return False

        picking_obj = self.pool['stock.picking']
        picking = picking_obj.browse(cr, uid, new_picking_id)

        for move in picking.move_lines:
            change_res = move._model.onchange_product_id(cr, uid, [move.id], move.product_id.id,
                                            move.location_id.id, move.location_dest_id.id,
                                            picking.partner_id.id)
            change_res['value']['product_qty'] = move.product_qty
            move._model.write(cr, uid, [move.id], change_res['value'])

        this.write({"state": "done", "picking_id": new_picking_id})

        # Escribimos el estado de las requested
        lines_requested = filter(lambda x: x.request_id, lines)
        lines_requested_ids = map(lambda x: x.request_id.id, lines_requested)
        material_line_obj.write(cr, uid, lines_requested_ids,
                {'state': 'done'})

        view_id = self.pool.get("ir.ui.view").search(cr, uid, [
            ('name', '=', 'view_suitcase_add_materials_wizard_form'),
            ('model', '=', 'suitcase.add.materials.wizard'),
            ('type', '=', 'form'),
        ])
        if view_id and isinstance(view_id, list):
            view_id = view_id[0]

        return {
            'name': 'Materials Changed',
            'res_model': 'suitcase.add.materials.wizard',
            'type': 'ir.actions.act_window',
            'views': [(view_id, 'form')],
            'view_id': view_id,
            'target': 'new',
            'res_id': ids[0],
            'context': context,
        }

    def continue_to_picking(self, cr, uid, ids, context=None):
        this = self.browse(cr, uid, ids[0])
        view_id = self.pool.get("ir.ui.view").search(cr, uid, [
            ('name', '=', 'stock.picking.form'),
            ('model', '=', 'stock.picking'),
            ('type', '=', 'form'),
        ])

        if view_id and isinstance(view_id, list):
            view_id = view_id[0]

        return {
            'name': 'Internal Picking List',
            'res_model': 'stock.picking',
            'type': 'ir.actions.act_window',
            'views': [(view_id, 'form')],
            'view_id': view_id,
            'view_mode': 'form',
            'target': 'current',
            'res_id': this.picking_id.id,
        }


class suitcase_add_materials_line(orm.TransientModel):
    _name = 'suitcase.add.materials.line'
    _description = 'Description'

    _columns = {
        'wiz_id': fields.many2one('suitcase.add.materials.wizard', 'Wizard'),
        'product_id': fields.many2one('product.product', 'Materials', required=True),
        'product_uom_id': fields.many2one('product.uom', string='UoM'),
        'product_uom_qty': fields.float(digits_compute=dp.get_precision('Product Unit of Measure'),
                                        string='Quantity'),
        'warehouse_id': fields.many2one('stock.warehouse', 'Warehouse'),
        'location_id': fields.many2one('stock.location', 'Location',
                                       domain=[('usage', '!=', 'view')]),
        # Guardamos la referencia para marcarlos como done
        'request_id': fields.many2one('suitcase.procure.materials', 'Requested'),
    }

    def _default_location_id(self, cr, uid, context=None):
        user = self.pool.get("res.users").browse(cr, uid, uid, context=context)
        warehouse_obj = self.pool.get("stock.warehouse")
        warehouse_id = warehouse_obj.search(cr, uid, [("company_id", "=", user.company_id.id)],
                                            order="id")
        warehouse = warehouse_obj.browse(cr, uid, warehouse_id[0], context)
        return warehouse.lot_stock_id.id

    def _default_warehouse_id(self, cr, uid, context=None):
        user = self.pool.get("res.users").browse(cr, uid, uid, context=context)
        warehouse_obj = self.pool.get("stock.warehouse")
        warehouse_id = warehouse_obj.search(cr, uid, [("company_id", "=", user.company_id.id)],
                                            order="id")
        return warehouse_id[0]

    _defaults = {
        "warehouse_id": _default_warehouse_id,
        "location_id": _default_location_id,
    }

    def onchange_product_id(self, cr, uid, ids, product_id, context=None):
        ret = {"value": {"product_uom_id": False}}
        if product_id:
            product = self.pool.get("product.product").browse(cr, uid, product_id, context=context)
            ret["value"]["product_uom_id"] = product.uom_id.id

        return ret
