# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2016 E-MIPS (http://www.e-mips.com.ar)
#    Copyright (c) 2016 Eynes (http://www.eynes.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General
#    Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from openerp.osv import fields, orm, osv
from openerp.tools.translate import _

import logging

_logger = logging.getLogger(__name__)


class AssignModuleToComponent(orm.TransientModel):
    _name = 'assign.module.to.component'
    _description = 'Assign Module To Component'

    def _system_from_form(self, cr, uid, context=None):
        if not context:
            return []
        system_obj = self.pool['service.system']
        system_id = context.get('active_ids', False)
        system = system_obj.browse(cr, uid, system_id, context)
        if not system_id or len(system_id) != 1:
            raise osv.except_osv(
                _('Error'), _('Expected only one sale in Wizard Call!'))
        if isinstance(system, list):
            system = system[0]
        return system

    def _default_components(self, cr, uid, context=None):
        system = self._system_from_form(cr, uid, context=context)
        return map(lambda x: x.id, system.component_ids)
        # return map(lambda x: x.id, filter(
        #     lambda x: not x.module_id, system.component_ids))

    def _default_remaining_modules(self, cr, uid, context=None):
        res = []
        system = self._system_from_form(cr, uid, context=context)
        module_ids = map(lambda x: x.module_id.id, system.component_ids)
        if system.system_type_id:
            res = map(lambda x: x.id, system.system_type_id.module_ids)
        return list(set(res) - set(module_ids))

    def onchange_component_ids(self, cr, uid, ids, system_id, component_ids,
                               context):
        component_obj = self.pool['service.component']
        res = {
            'value': {}
        }
        mod = False
        system_obj = self.pool['service.system']
        system = system_obj.browse(cr, uid, system_id, context=context)
        if system.system_type_id:
            remains = map(lambda x: x.id, system.system_type_id.module_ids)
        edited = []
        xcomponent_ids = map(lambda x: x[1], component_ids)
        components = component_obj.browse(cr, uid, xcomponent_ids, context)
        used = filter(lambda x: x, map(lambda x: x.module_id.id, components))
        removed_from_comp = []
        for component_com in component_ids:
            if component_com and len(component_com) == 3 and \
                    isinstance(component_com[2], dict) and \
                    'module_id' in component_com[2]:
                mod = True
                if component_com[2]['module_id'] is False or \
                        component_com[2]['module_id'] != component_obj. \
                        browse(cr, uid, component_com[1]).module_id.id:
                    removed_from_comp.append(component_com[1])
                if component_com[2]['module_id'] and \
                        component_com[2]['module_id'] in remains:
                    edited.append(component_com[2]['module_id'])
        removed_comps = component_obj.browse(cr, uid, removed_from_comp,
                                             context)
        removed = filter(lambda x: x, map(lambda x: x.module_id.id,
                                          removed_comps))
        used = list(set(used) - set(removed))
        if mod:
            new_remains = list(set(remains) - set(used + edited))
            res['value']['remaining_module_ids'] = [(6, 0, new_remains)]
        return res

    def save_assignation(self, cr, uid, ids, context=None):
        return True

    _columns = {
        'system_id': fields.many2one('service.system', 'System'),
        'remaining_module_ids': fields.many2many(
            'service.module', 'service_assign_module_module_rel',
            'assing_module_wiz_id', 'module_id', string='Remaining Modules'),
        'component_line_ids': fields.many2many(
            'service.component', 'service_assign_module_component_rel',
            'assign_module_wiz_id', 'component_id', string='Components'),
    }

    _defaults = {
        'system_id': lambda s, cr, uid, c: s._system_from_form(
            cr, uid, c).id,
        'component_line_ids': lambda s, cr, uid, c: s._default_components(
            cr, uid, c),
        'remaining_module_ids': lambda s, cr, uid, c:
            s._default_remaining_modules(cr, uid, c),
    }
